@@ -5,17 +5,16 @@ from shapely.geometry import Point
 
 
 class CRS:
-
     def setup(self):
-        nybb = read_file(datasets.get_path('nybb'))
-        self.long_nybb = GeoDataFrame(pd.concat(10 * [nybb]),
-                                      crs=nybb.crs)
+        nybb = read_file(datasets.get_path("nybb"))
+        self.long_nybb = GeoDataFrame(pd.concat(10 * [nybb]), crs=nybb.crs)
 
         num_points = 20000
         longitudes = np.random.rand(num_points) - 120
         latitudes = np.random.rand(num_points) + 38
-        self.point_df = GeoSeries([Point(x, y) for (x, y)
-                                  in zip(longitudes, latitudes)])
+        self.point_df = GeoSeries(
+            [Point(x, y) for (x, y) in zip(longitudes, latitudes)]
+        )
         self.point_df.crs = {"init": "epsg:4326"}
 
     def time_transform_wgs84(self):
