@@ -2,16 +2,18 @@ import geopandas as gpd
 import numpy as np
 import pandas as pd
 import pytest
+import shapely
 from packaging.version import Version
 
 folium = pytest.importorskip("folium")
 branca = pytest.importorskip("branca")
 matplotlib = pytest.importorskip("matplotlib")
 mapclassify = pytest.importorskip("mapclassify")
+geodatasets = pytest.importorskip("geodatasets")
 
-import matplotlib.cm as cm  # noqa
-import matplotlib.colors as colors  # noqa
-from branca.colormap import StepColormap  # noqa
+from matplotlib import cm
+from matplotlib import colors
+from branca.colormap import StepColormap
 
 BRANCA_05 = Version(branca.__version__) > Version("0.4.2")
 FOLIUM_G_014 = Version(folium.__version__) > Version("0.14.0")
@@ -22,6 +24,7 @@ class TestExplore:
         self.nybb = gpd.read_file(gpd.datasets.get_path("nybb"))
         self.world = gpd.read_file(gpd.datasets.get_path("naturalearth_lowres"))
         self.cities = gpd.read_file(gpd.datasets.get_path("naturalearth_cities"))
+        self.chicago = gpd.read_file(geodatasets.get_path("geoda.chicago_commpop"))
         self.world["range"] = range(len(self.world))
         self.missing = self.world.copy()
         np.random.seed(42)
@@ -173,6 +176,7 @@ class TestExplore:
         assert '"fillColor":"#5ec962"' in out_str
         assert '"fillColor":"#fde725"' in out_str
         assert '"fillColor":"#440154"' in out_str
+
         # custom k
         m = self.world.explore(column="pop_est", scheme="naturalbreaks", k=3)
         out_str = self._fetch_map_string(m)
@@ -180,6 +184,18 @@ class TestExplore:
         assert '"fillColor":"#fde725"' in out_str
         assert '"fillColor":"#440154"' in out_str
 
+        # UserDefined overriding default k
+        m = self.chicago.explore(
+            column="POP2010",
+            scheme="UserDefined",
+            classification_kwds={"bins": [25000, 50000, 75000, 100000]},
+        )
+        out_str = self._fetch_map_string(m)
+        assert '"fillColor":"#fde725"' in out_str
+        assert '"fillColor":"#35b779"' in out_str
+        assert '"fillColor":"#31688e"' in out_str
+        assert '"fillColor":"#440154"' in out_str
+
     def test_categorical(self):
         """Categorical maps"""
         # auto detection
@@ -325,11 +341,11 @@ class TestExplore:
     def test_style_kwds(self):
         """Style keywords"""
         m = self.world.explore(
-            style_kwds=dict(fillOpacity=0.1, weight=0.5, fillColor="orange")
+            style_kwds={"fillOpacity": 0.1, "weight": 0.5, "fillColor": "orange"}
         )
         out_str = self._fetch_map_string(m)
         assert '"fillColor":"orange","fillOpacity":0.1,"weight":0.5' in out_str
-        m = self.world.explore(column="pop_est", style_kwds=dict(color="black"))
+        m = self.world.explore(column="pop_est", style_kwds={"color": "black"})
         assert '"color":"black"' in self._fetch_map_string(m)
 
         # custom style_function - geopandas/issues/2350
@@ -348,14 +364,12 @@ class TestExplore:
         # two lines with formatting instructions from style_function.
         # make sure each passes test
         assert all(
-            [
-                ('"fillColor":"green"' in t and '"color":"white"' in t)
-                or ('"fillColor":"red"' in t and '"color":"black"' in t)
-                for t in [
-                    "".join(line.split())
-                    for line in m._parent.render().split("\n")
-                    if "return" in line and "color" in line
-                ]
+            ('"fillColor":"green"' in t and '"color":"white"' in t)
+            or ('"fillColor":"red"' in t and '"color":"black"' in t)
+            for t in [
+                "".join(line.split())
+                for line in m._parent.render().split("\n")
+                if "return" in line and "color" in line
             ]
         )
 
@@ -426,7 +440,7 @@ class TestExplore:
         m = self.world.explore(
             tooltip=True,
             popup=False,
-            tooltip_kwds=dict(aliases=[0, 1, 2, 3, 4, 5], sticky=False),
+            tooltip_kwds={"aliases": [0, 1, 2, 3, 4, 5], "sticky": False},
         )
         out_str = self._fetch_map_string(m)
         assert (
@@ -440,7 +454,7 @@ class TestExplore:
         m = self.world.explore(
             tooltip=False,
             popup=True,
-            popup_kwds=dict(aliases=[0, 1, 2, 3, 4, 5]),
+            popup_kwds={"aliases": [0, 1, 2, 3, 4, 5]},
         )
         out_str = self._fetch_map_string(m)
         assert (
@@ -454,8 +468,8 @@ class TestExplore:
         m = self.world.explore(
             tooltip=True,
             popup=True,
-            tooltip_kwds=dict(labels=False),
-            popup_kwds=dict(labels=False),
+            tooltip_kwds={"labels": False},
+            popup_kwds={"labels": False},
         )
         out_str = self._fetch_map_string(m)
         assert "<th>${aliases[i]" not in out_str
@@ -483,7 +497,7 @@ class TestExplore:
         for s in strings:
             assert s in out_str
 
-        m = self.cities.explore(marker_kwds=dict(radius=5, fill=False))
+        m = self.cities.explore(marker_kwds={"radius": 5, "fill": False})
         strings = ['"radius":5', '"fill":false', "CircleMarker(latlng,opts)"]
         out_str = self._fetch_map_string(m)
         for s in strings:
@@ -556,10 +570,10 @@ class TestExplore:
         m = self.missing.explore("pop_est")
         assert '"fillColor":null' in self._fetch_map_string(m)
 
-        m = self.missing.explore("pop_est", missing_kwds=dict(color="red"))
+        m = self.missing.explore("pop_est", missing_kwds={"color": "red"})
         assert '"fillColor":"red"' in self._fetch_map_string(m)
 
-        m = self.missing.explore("continent", missing_kwds=dict(color="red"))
+        m = self.missing.explore("continent", missing_kwds={"color": "red"})
         assert '"fillColor":"red"' in self._fetch_map_string(m)
 
     def test_categorical_legend(self):
@@ -590,13 +604,13 @@ class TestExplore:
         assert quoted_in("text('range')", out_str)
 
         m = self.world.explore(
-            "range", legend=True, legend_kwds=dict(caption="my_caption")
+            "range", legend=True, legend_kwds={"caption": "my_caption"}
         )
         out_str = self._fetch_map_string(m)
         assert "attr(\"id\",'legend')" in out_str
         assert quoted_in("text('my_caption')", out_str)
 
-        m = self.missing.explore("pop_est", legend=True, missing_kwds=dict(color="red"))
+        m = self.missing.explore("pop_est", legend=True, missing_kwds={"color": "red"})
         out_str = self._fetch_map_string(m)
         assert "red'></span>NaN" in out_str
 
@@ -604,7 +618,7 @@ class TestExplore:
         m = self.world.explore(
             "pop_est",
             legend=True,
-            legend_kwds=dict(scale=False),
+            legend_kwds={"scale": False},
             scheme="Headtailbreaks",
         )
         out_str = self._fetch_map_string(m)
@@ -645,7 +659,7 @@ class TestExplore:
         import re
 
         # linear
-        m = self.world.explore("pop_est", legend_kwds=dict(max_labels=3))
+        m = self.world.explore("pop_est", legend_kwds={"max_labels": 3})
         out_str = self._fetch_map_string(m)
         tick_str = re.search(r"tickValues\(\[[\',\,\.,0-9]*\]\)", out_str).group(0)
         assert (
@@ -655,13 +669,13 @@ class TestExplore:
 
         # scheme
         m = self.world.explore(
-            "pop_est", scheme="headtailbreaks", legend_kwds=dict(max_labels=3)
+            "pop_est", scheme="headtailbreaks", legend_kwds={"max_labels": 3}
         )
         out_str = self._fetch_map_string(m)
         assert "tickValues([140.0,'',184117213.1818182,'',1382066377.0,''])" in out_str
 
         # short cmap
-        m = self.world.explore("pop_est", legend_kwds=dict(max_labels=3), cmap="tab10")
+        m = self.world.explore("pop_est", legend_kwds={"max_labels": 3}, cmap="tab10")
         out_str = self._fetch_map_string(m)
 
         tick_str = re.search(r"tickValues\(\[[\',\,\.,0-9]*\]\)", out_str).group(0)
@@ -746,8 +760,8 @@ class TestExplore:
             column="pop_est",
             legend=True,
             scheme="naturalbreaks",
-            missing_kwds=dict(color="red", label="missing"),
-            legend_kwds=dict(colorbar=False, interval=True),
+            missing_kwds={"color": "red", "label": "missing"},
+            legend_kwds={"colorbar": False, "interval": True},
         )
         out_str = self._fetch_map_string(m)
 
@@ -767,8 +781,8 @@ class TestExplore:
             column="pop_est",
             legend=True,
             scheme="naturalbreaks",
-            missing_kwds=dict(color="red", label="missing"),
-            legend_kwds=dict(colorbar=False, interval=False),
+            missing_kwds={"color": "red", "label": "missing"},
+            legend_kwds={"colorbar": False, "interval": False},
         )
         out_str = self._fetch_map_string(m)
 
@@ -789,7 +803,7 @@ class TestExplore:
             legend=True,
             scheme="naturalbreaks",
             k=5,
-            legend_kwds=dict(colorbar=False, labels=["s", "m", "l", "xl", "xxl"]),
+            legend_kwds={"colorbar": False, "labels": ["s", "m", "l", "xl", "xxl"]},
         )
         out_str = self._fetch_map_string(m)
 
@@ -802,8 +816,8 @@ class TestExplore:
             column="pop_est",
             legend=True,
             scheme="naturalbreaks",
-            missing_kwds=dict(color="red", label="missing"),
-            legend_kwds=dict(colorbar=False, fmt="{:.0f}"),
+            missing_kwds={"color": "red", "label": "missing"},
+            legend_kwds={"colorbar": False, "fmt": "{:.0f}"},
         )
         out_str = self._fetch_map_string(m)
 
@@ -836,7 +850,7 @@ class TestExplore:
         assert '"fillOpacity":0.75' in out_str
 
         m = self.nybb.explore(
-            highlight=True, highlight_kwds=dict(fillOpacity=1, color="red")
+            highlight=True, highlight_kwds={"fillOpacity": 1, "color": "red"}
         )
         out_str = self._fetch_map_string(m)
 
@@ -914,12 +928,30 @@ class TestExplore:
 
         # check that folium and leaflet Map() parameters can be passed
         m = self.world.explore(
-            zoom_control=False, map_kwds=dict(dragging=False, scrollWheelZoom=False)
+            zoom_control=False, map_kwds={"dragging": False, "scrollWheelZoom": False}
         )
         check()
         with pytest.raises(
             ValueError, match="'zoom_control' cannot be specified in 'map_kwds'"
         ):
             self.world.explore(
-                map_kwds=dict(dragging=False, scrollWheelZoom=False, zoom_control=False)
+                map_kwds={
+                    "dragging": False,
+                    "scrollWheelZoom": False,
+                    "zoom_control": False,
+                }
             )
+
+    def test_none_geometry(self):
+        # None and empty geoms are dropped prior plotting
+        df = self.nybb.copy()
+        df.loc[0, df.geometry.name] = None
+        m = df.explore()
+        self._fetch_map_string(m)
+
+    def test_empty_geometry(self):
+        # None and empty geoms are dropped prior plotting
+        df = self.nybb.copy()
+        df.loc[0, df.geometry.name] = shapely.Point()
+        m = df.explore()
+        self._fetch_map_string(m)
